@@ -1,6 +1,8 @@
 import os
 import random
 import string
+import networkx as nx
+
 from contextlib import redirect_stdout
 from io import StringIO
 
@@ -13,13 +15,10 @@ def new_name():
     return result_str
 
 
-def create_app(interpreter, client_instance):
+def create_app(interpreter):
     # create and configure the app
     app = Flask(__name__, instance_relative_config=True)
     app.root_path = os.path.dirname(os.path.abspath(__file__)[:-6])
-    app.config.from_mapping(
-        SECRET_KEY='dev'
-    )
 
     interpreter.create_cell(["create_cell", "root", "python", "n"])
 
@@ -95,13 +94,17 @@ def create_app(interpreter, client_instance):
     @app.route("/recursion_check/", methods=["POST"])
     def recursion_check():
         data = request.get_json()
-        cell_name = data['cell_name'].strip()
+        first = data['first'].strip()
+        second = data['second'].strip()
 
         nodes, _, edge_names = interpreter.graph.get_all_cells_edges()
 
-        for e in edge_names:
-            if e[0] == cell_name:
-                return "500"
+        if interpreter.graph.name_to_idx(second) == 0:
+            return "Can't link to root cell"
+
+        if interpreter.graph.name_to_idx(second) in \
+                nx.ancestors(interpreter.graph.graph, interpreter.graph.name_to_idx(first)):
+            return "Cycles are not allowed"
 
         return "200"
 
@@ -130,9 +133,7 @@ def create_app(interpreter, client_instance):
         if first == second:
             return "500"
 
-        interpreter.link(['link', first, second])
-
-        return "200"
+        return interpreter.link(['link', first, second])
 
     @app.route("/bfs_execute/", methods=["POST"])
     def bfs_execute():
@@ -155,7 +156,7 @@ def create_app(interpreter, client_instance):
         satx_text = interpreter.graph.get_satx_as_txt()
         satx_text += "\n<!--SATYRN_POSITIONING_START-->"
 
-        for i in range(len(names)):
+        for i, _ in enumerate(names):
             satx_text += "\n" + names[i] + lefts[i] + " " + tops[i]
 
         satx_text += "\n<!--SATYRN_POSITIONING_END-->"
@@ -303,9 +304,19 @@ def create_app(interpreter, client_instance):
 
         with interpreter.lock:
             cell = interpreter.graph.get_cell(name)
+            if not cell:
+                return "201"
             cell.top = top
             cell.left = left
 
         return "200"
 
+    @app.route("/get_layer/", methods=["POST"])
+    def get_layer():
+        cell_name = request.get_json()['cell_name']
+
+        out = interpreter.get_layer(cell_name)
+
+        return str(out) if out > 0 else " "
+
     return app
